-- Test payment_method
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(30);

set search_path to camper, auth, public;

select has_table('payment_method');
Select has_pk('payment_method' );
select table_privs_are('payment_method', 'guest', array []::text[]);
select table_privs_are('payment_method', 'employee', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_method', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_method', 'authenticator', array []::text[]);

select has_column('payment_method', 'payment_method_id');
select col_is_pk('payment_method', 'payment_method_id');
select col_type_is('payment_method', 'payment_method_id', 'integer');
select col_not_null('payment_method', 'payment_method_id');
select col_hasnt_default('payment_method', 'payment_method_id');

select has_column('payment_method', 'company_id');
select col_is_fk('payment_method', 'company_id');
select fk_ok('payment_method', 'company_id', 'company', 'company_id');
select col_type_is('payment_method', 'company_id', 'integer');
select col_not_null('payment_method', 'company_id');
select col_hasnt_default('payment_method', 'company_id');

select has_column('payment_method', 'name');
select col_type_is('payment_method', 'name', 'text');
select col_not_null('payment_method', 'name');
select col_hasnt_default('payment_method', 'name');

select has_column('payment_method', 'instructions');
select col_type_is('payment_method', 'instructions', 'text');
select col_not_null('payment_method', 'instructions');
select col_hasnt_default('payment_method', 'instructions');


set client_min_messages to warning;
truncate payment_method cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into payment_method(payment_method_id, company_id, name, instructions)
values (222, 2, 'Cash', 'Payment in cash.')
     , (223, 2, 'Wire Transfer', 'Please, transfer money to my Nigerian bank')
     , (444, 4, 'Pigeon', 'Send money via carrier pigeon.')
;

prepare payment_method_data as
select company_id, name, instructions
from payment_method;

set role employee;
select is_empty('payment_method_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');
select bag_eq(
	'payment_method_data',
	$$ values ( 2, 'Cash', 'Payment in cash.' )
	        , ( 2, 'Wire Transfer', 'Please, transfer money to my Nigerian bank' )
	$$,
	'Should only list payment methods of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'co4');
select bag_eq(
	'payment_method_data',
	$$ values (4, 'Pigeon', 'Send money via carrier pigeon.' )
	$$,
	'Should only list payment methods of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie', 'co4');
select throws_ok(
	'payment_method_data',
	'42501', 'permission denied for table payment_method',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into payment_method (company_id, name, instructions)
	values (2, '  ', 'atasht')
	$$,
	'23514', 'new row for relation "payment_method" violates check constraint "name_not_empty"',
	'Should not allow payment methods with blank name'
);


select *
from finish();

rollback;

